package algorithm;

import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;

import javax.swing.JFrame;
import javax.swing.JTree;
import javax.swing.SwingUtilities;

import graph.LayeredGraphNode;
import processor.PseudoCodeNode;
import processor.PseudoCodeProcessor;
import processor.PseudoCodeProcessor.CodeStatus;

/**
 * represents an algorithm that can be run by an {@link PseudoCodeProcessor}
 * @author kolja
 *
 */
public abstract class Algorithm extends Thread {

    protected AnimationController ac;
    protected LayeredGraphNode graph;
    private JFrame view;
    protected PseudoCodeNode root;
    protected PseudoCodeProcessor processor;
    private boolean renderImage = false;

    /**
     * creates an {@link Algorithm} and initializes its fields
     * @param controller The controller that this algorithm is controlled by
     * @param graph The graph that this algorithm works with
     * @param view The view where the progress is displayed
     */
    public Algorithm( AnimationController controller, LayeredGraphNode graph, JFrame view )
    {
        this.ac = controller;
        this.graph = graph;
        this.view = view;
        root = null;
        renderImage = true;
    }

    private synchronized void update()
    {
        if( renderImage )
        {
            renderImage = false;
            SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    for( ComponentListener l : view.getComponentListeners() )
                    {
                        l.componentResized( new ComponentEvent(view, 0) );
                        synchronized( Algorithm.this )
                        {
                            renderImage = true;
                        }
                    }
                }
            });
        }
    }

    @Override
    public void run()
    {
        while( true ) // if this loop would end we could not undo steps any more
        {
        	CodeStatus status = null;
            try {
                if( processor != null )
                {
                    Action action = ac.getNextAction();
                    graph.unselectGraph();
                    switch( action )
                    {
                    case FORWARD:
                    	status = processor.forwardStep();
                        break;
                    case FORWARD_OUT:
                        status = processor.forwardStepOut();
                        break;
                    case FORWARD_OVER:
                        status = processor.forwardStepOver();
                        break;
                    case BACKWARD:
                        status = processor.backwardStep();
                        break;
                    case BACKWARD_OUT:
                        status = processor.backwardStepOut();
                        break;
                    case BACKWARD_OVER:
                        status = processor.backwardStepOver();
                        break;
                    default:
                        assert false;
                        break;
                    }
                }
                else
                    status = CodeStatus.FINISHED;
            } catch (InterruptedException e) {
                e.printStackTrace();
                return;
            }
            update();
            if( status == CodeStatus.FINISHED )
            {
        		ac.setContinuous( false );
        		ac.setNextAction( null );
            }
        }
    }

    public abstract PseudoCodeNode createPseudocodeTree( JTree tree );
    
    public PseudoCodeProcessor getProcessor() {
        return processor;
    }
    
    /**
     * returns a string with debugging information about the current status algorithm
     * @return some information
     */
    public String getDebugString()
    {
        if( processor == null )
            return "";
        return processor.getDebugOutput();
    }
}
